\name{bea}
\alias{bea}
\title{
Bond Energy Algorithm 
}
\usage{
bea(a, istart = 0, jstart = 0)
}
\description{
  Permutes  rows  and columns of an array, in order to maximize
  proximity of large-valued array elements. 
}
\arguments{
\item{a}{
data matrix to be analyzed.  The rows could represent observations and the
columns variables (a two-way, two-mode array); or the rows and columns could
both represent observations (a two-way, one-mode array).  Missing values are
not supported.
}
\item{istart}{
sequence number of first row to be placed; if not specified, then a row is 
arbitrarily chosen.
}
\item{jstart}{
sequence number of first column to be placed; if not specified, then a row is
arbitrarily chosen.
}}
\value{
list describing the reordering carried out:

\item{b}{
row and column permuted array.
}
\item{ib}{
permutation of rows.
}
\item{jb}{
permutation of columns.
}
\item{e}{
\code{bond energy} of the permuted matrix, \code{b}.
}}
\section{Note}{
This is a \code{non-destructive} approach to analyzing data, insofar as the 
original data is not altered; it is only rearranged, in order to highlight
potentially interesting aspects of the data.  Subsequent runs of this 
routine may improve the bond energy (see example below).  
}
\section{Method}{
A row is arbitrarily placed; then rows are positioned one by one.  When this
is completed, the columns are treated similarly.  The overall procedure 
amounts to two approximate traveling salesman problems, - one on the rows and 
one on the columns.  The so-called \code{best insertion} strategy is used: rows 
(or columns) are inserted into the current permuted list of rows (or columns).


To have repeatable results, control the random number generator used for the
initial selection of rows and columns with the \code{set.seed} command,
before calling the \code{bea} function.  


No methods are currently defined for the objects produced by the \code{bea}
function.
}
\section{Background}{
This simple method has been used in operations research, production 
engineering, marketing, and various other fields.  Arabie and Hubert (1990)
recommend that it be used with ratio scale data; and they question its use 
with non-binary data if the objective is to find a seriation or 
one-dimensional ordering of rows and columns.
}
\references{


W.T. McCormick, P.J. Schweitzer and T.W. White, 
\code{Problem decomposition and data reorganization by a clustering technique}, 
Operations Research, 
vol. 20, pp. 993-1009, Sept./Oct. 1972.


P. Arabie and L.J. Hubert, 
\code{The bond energy algorithm revisited}, 
IEEE Transactions on Systems, Man, and Cybernetics, 
vol. 20, pp. 268-274, 1990.


P. Arabie, S. Schleutermann, J. Daws and L. Hubert, 
`Marketing applications of sequencing and partitioning of nonsymmetric 
and/or two-mode matrices', 
in W. Gaul and M. Schader, Eds., 
Data Analysis, Decision Support, \& Expert Knowledge Representation in 
Marketing,
Springer Verlag, 1988, pp. 215-224.
}
\examples{
data(USArrests)
USArrests <- as.matrix(USArrests)
run1 <- bea(USArrests)
# look at energy:
run1$e
# Redo, on basis of first run, to see if energy increases:
run2 <- bea(run1$b)
run2$e
# Remark: to have repeatable results, issue the `set.seed' before the `bea'
# command.  Now, reorder the rows and the columns once more:
run3 <- bea(run2$b)
run3$e
# Of course, sequencing of rows and columns of each run is relative to 
# the array which was input.  Get `net' ordering of rows after third run:
run1$ib[run2$ib[run3$ib]]
# Plot the output as images:
image(1:nrow(USArrests), 1:ncol(USArrests), USArrests)
image(1:nrow(USArrests), 1:ncol(USArrests), bea(USArrests)$b)
# Remark: one could split the screen, and use color in such displays of
# the data.
}
\keyword{multivariate}
\keyword{cluster}
% Converted by Sd2Rd version 0.2-a5.
\eof
\name{ca}
\alias{ca}
\title{
Correspondence Analysis
}
\description{
  Finds a new coordinate system for multivariate data such that the first
coordinate has maximal inertia, the second coordinate has maximal inertia
subject to being orthogonal to the first, etc.  Compared to Principal 
Components Analysis, each row and column point has an associated mass 
(related to the row or column totals); and the chi-squared distance takes the 
place of the Euclidean distance.  The issue of how to code the input data
is important: this takes the place of input data transformation in PCA.
}
\usage{
ca(a, nf = 7)
}
\arguments{
\item{a}{
data matrix to be decomposed, the rows representing observations and the
columns variables.
}
\item{nf}{
number of factors or axes to be sought; default 7.  
}}
\value{
a list containg the following results:

\item{rproj}{
projections of row points on the factors.
}
\item{cproj}{
projections of column points on the factors.
}
\item{evals}{
eigenvalues associated with the new factors. These provide figures of merit for
the "inertia explained" by the factors.  They are usually quoted in terms
of percentage of the total, or in terms of cumulative percentage of the total.
}
\item{rcntr}{
contributions of observations to the factors.  The contributions are mass times
projection (on the factor) squared.  Since contributions take account of the 
mass, they more accurately indicate influential observations for the 
interpretation of the factor, compared to the projections alone.
}
\item{ccntr}{
contributions of variables to the factors. See above remark concerning 
row contributions.
}}
\section{Note}{
Very small negative eigenvalues, if they arise, are an artifact of the SVD
algorithm used, and are to be treated as zero.
}
\section{Method}{
A singular value decomposition is carried out.  
}
\section{Background}{
Correspondence analysis defines the axis which provides the best fit
to both the row points and the column points.  A second axis is determined
which best fits the data subject to being orthogonal to the first.  Third and
subsequent axes are similarly found.  Best fit is in the least squares sense,
relative to the chi-squared distance.  This can be viewed as a weighted
Euclidean distance between \code{profiles}.


The question of \code{coding} of input data is an important one.  For instance,
in a matrix of scores, one might wish to adjoin extra columns to the input
matrix such that both the initial score, and the maximum score minus it,
are included in the observation's set of values.  Note that this has the
effect that all row masses are equal.  Hence the variables alone are 
differentially weighted.  This is known as \code{doubling} the observations.
In the case of binary data, such coding is known as 
\code{complete disjunctive form}.  


Other forms of input data for which correspondence analysis can be used include
frequencies, or contingency-type data.  In this case, the totaled chi-squared
distances of all (row or column) points from the origin is the familiar
chi-squared statistic. Hence the graphical output of correspondence analysis
allows assessment of departure from a null hypothesis of no dependence of 
rows and columns.


Supplementary rows or columns are projected into the factor space, after
carrying out a correspondence analysis.  That is to say, such row or 
column profiles are assumed to have zero mass, and their projections are
to be found under such an assumption.  Functions \code{supplr} and \code{supplc} may
be used for this purpose.  Supplementary rows or columns are of a different
nature compared to the basis data analyzed (e.g. sex in the context of a 
questionnaire); or they are rows or columns which, one suspects, would 
untowardly influence the definition of the factors.
}
\references{
Extensive works of J.-P. Benzecri including
Correspondence Analysis Handbook
Marcel Dekker, Basel, 1992.


M.J. Greenacre,
Theory and Applications of Correspondence Analysis
Academic Press, New York, 1984.


L. Lebart, A. Morineau and K.M. Warwick,
Multivariate Descriptive Statistical Analysis
Wiley, New York, 1984.


S. Nishisato, 
Analysis of Categorical Data: Dual Scaling and Its Applications
University of Toronto Press, Toronto, 1980.


(An extensive annotated bibliography is to be found in Greenacre.)
}
\seealso{
Supplementary rows and columns: \code{\link{supplr}}, \code{\link{supplc}}.  Initial data coding:
\code{\link{flou}}, \code{\link{logique}}.  Other related functions: \code{\link{pca}}, \code{\link{prcomp}}, \code{\link{cancor}}, 
\code{\link{sammon}}, \code{\link{cmdscale}}.
}
\examples{
data(USArrests)
USArrests <- as.matrix(USArrests)
corr <- ca(USArrests)
# plot of first and second factors
plot(corr$rproj[,1], corr$rproj[,2],type="n")
text(corr$rproj[,1], corr$rproj[,2])
# check of row contributions
corr$rcntr
#
# Fuzzy coding of input variables
a.fuzz <- flou(USArrests[,1])
b.fuzz <- flou(USArrests[,2])
c.fuzz <- flou(USArrests[,3])
d.fuzz <- flou(USArrests[,4])
newdata <- cbind(a.fuzz, b.fuzz, c.fuzz, d.fuzz)
ca.newdata <- ca(newdata)
}
\keyword{multivariate}
\keyword{algebra}
% Converted by Sd2Rd version 0.2-a5.


\eof
\name{flou}
\alias{flou}
\title{
Fuzzy Coding (3-Way)
}
\description{
  Simple fuzzy, or piecewise linear, coding of a vector: each value in the 
  vector is replaced by a 1 (if it is above or equal to the 67th quantile),
  by a 0 (if it is below or equal to the 33rd quantile), and by a linearly
  interpolated value between 0 and 1 (if it lies between the 33rd and 67th
  quantiles).  
}
\usage{
flou(x)
}
\arguments{
\item{x}{
real-valued vector, with no missing values.
}}
\value{
matrix of \code{length(a)} rows, and two columns.  The first column contains the
fuzzily coded values of \code{a}, and the second column contains their 
complements.  Hence each row of this returned matrix necessarily sums to 1.
}
\section{Background}{
This form of coding is suitable for a subsequent correspondence analysis.
When all variable have been fuzzily (or logically) coded, the row masses 
(proportional to the row sums) are identical.  
}
\references{
J.-P. Benzecri
Correspondence Analysis Handbook
Marcel Dekker, Basel, 1992.


F.J. Gallego,
Codage flou en analyse des correspondances,
Les Cahiers de l'Analyse des Donnees
vol. VII, 413-430, 1982
}
\seealso{
\code{\link{logique}}, \code{\link{ca}}, \code{\link{supplr}}, \code{\link{supplc}}.  
}
\examples{
data(USArrests)
USArrests <- as.matrix(USArrests)
# Fuzzy coding of input variables
a.fuzz <- flou(USArrests[,1])
b.fuzz <- flou(USArrests[,2])
c.fuzz <- flou(USArrests[,3])
d.fuzz <- flou(USArrests[,4])
newdata <- cbind(a.fuzz, b.fuzz, c.fuzz, d.fuzz)
ca.newdata <- ca(newdata)
}
\keyword{multivariate}
\keyword{algebra}
% Converted by Sd2Rd version 0.2-a5.


\eof
\name{hierclust}
\alias{hierclust}
\alias{hc}
\title{
  Hierarchical Clustering
}
\usage{
hierclust(a, method = 1, bign = FALSE, diagnostics = FALSE, sim = " ",
             movie = FALSE, option = "thru", alpha = 1.0, repres = "chull", 
             show = "change")
}
\description{
  Agglomerates the closest pair of observations, and replaces them with
  a cluster.  Agglomerates the next closest pair of observations or clusters.
  Continues through `n-1' agglomerations, assuming there are `n' observations.
}
\arguments{
  \item{a}{
    data matrix to be analyzed, the rows representing observations and the
    columns variables; or, alternatively, lower-diagonal dissimilarity
    matrix in vector or list format.  In latter case, the number of values is
    \code{n(n-1)/2} where \code{n} is the number of observations.
  }
  \item{method}{
    First possibility: integer taking the value between 1 and 7.  These
    methods correspond to: 
    Ward's minimum variance or error sum of squares method (1); single linkage
    or nearest neighbor method (2); complete linkage or diameter (3); average
    linkage, group average, or UPGMA method (4); McQuitty's or WPGMA method
    (5); median, Gower's or WPGMC method (6); and centroid or UPGMC method (7).
    If the \code{movie} option is used, only \code{method}s 1 to 4 are
    supported.
    
    Second possibility: string \code{compact} (default; complete linkage), 
    \code{average} (average linkage), or \code{connected} (single linkage).}
  \item{sim}{
    structure giving similarities rather than dissimilarities.  This can either
    be a symmetric matrix or a vector with a \code{Size} attribute.
    Only one of dissimilarities or similarities may be specified (or, of
    course, as an alternative, a data matrix.
  }
  \item{bign}{
    Is \code{n} big?  If storage is problemsome, a different
    implementation of the  Ward criterion may be tried.  This determines
    dissimilarities on the fly, and 
    hence requires \code{O(n)} storage.
  }
\item{diagnostics}{
show, or suppress, some information directed to the command window. 
Suppression is useful if functional programming capabilities are availed of,
leading to lazy evaluation; this often results in multiple evaluations, 
with consequent repetition of diagnostic information.
}
\item{movie}{
whether interactive display of construction of hierarchy is wanted.  This
necessitates an open plot window.  It does not produce an output object.  
All remaining arguments apply only in the case of the \code{movie=TRUE}.
}
\item{option}{
(Only for \code{movie}.) "thru" (default) or "prompt": Go right through all n-1 
agglomerations, or ask
the user whether to continue at each agglomeration?  Here is what we
recommend: On the first occasion, go right through.  Check out the cluster
criterion values reported on in the command window.  Then using the 'prompt'
option, go as far as the desired partition, and print it out.
}
\item{alpha}{
(Only for \code{movie}.) 1.0 (default) or 0.0 < alpha <= 1.0.
Only used by the minimum variance agglomerative criterion. Alpha downweights
the variance component on the principal axis of the cluster, and thereby
redefines the cluster criterion such that linear clusters are preferred.  A
value of alpha = 1.0 gives exactly the traditional Ward's minimum variance
method.  See Murtagh and Raftery (1984) for more on this altered criterion.
}
\item{repres}{
(Only for \code{movie}.) repres = "chull" (default), "lines", "gauss":
Representation of clusters: For input data which has dimensionality greater
than 2, only "chull" is allowed (since the representations were found to be
too problematic, given that the clustering takes place in m-dimensions, and
the the representation is necessarily a 2-dimensional projection of this).
Representation "chull" = convex hull, or straight line in case
of collinear points.  (We test for collinearity using principal
components analysis.)
Representation "lines" = lines of best fit (least squares fit is used,
and the extremities are defined by the intersection of the min. and max. x
values on this LS fit line). "gauss" = 1-sigma circles.  Side-effect of
latter: warning messages indicate whenever circles overlap plot boundaries.
}
\item{show}{
(Only for \code{movie}.) "change" or "all":
In this method's present implementation, the clusters at each stage are
\code{highlighted} by means of a different symbol.  Option show = "change" has
this highlighting changed at each agglomeration, so that only the most recent
agglomerands are shown.  Option show = "all" does not turn off the
highlighting in this way.
}}
\value{
When \code{movie} is not specified in the first variant, this is a 
list describing the hierarchical clustering of the observations.
The \code{movie} option is considerably slower than alternative options, due
to the processing involved.  Hence its use is recommended only on small
data sets (less than 100 observations).

\item{merge}{
an \code{n-1} by 2 matrix. Row \code{i} of \code{merge} describes the merging of clusters
at step \code{i} of the clustering.  If an element \code{j} in the row is negative,
then observation \code{-j} was merged at this stage.  If \code{j} is positive 
then the merge
was with the cluster formed at the (earlier) stage \code{j} of the algorithm. Thus
negative entries in \code{merge} indicate agglomerations of singletons, and 
positive entries indicate agglomerations of non-singletons.
}
\item{height}{
a set of \code{n-1} non-decreasing real values.  The
clustering "height": that is, the value of the criterion associated with
the clustering \code{method} for the particular agglomeration.
}
\item{order}{
a vector giving the permutation of the original observations suitable for 
plotting,
in the sense that a cluster plot using this ordering and matrix \code{merge} will
not have crossings of the branches.


In hierarchical cluster displays, a decision is needed at each merge to
specify which subtree should go on the left and which on the right.  Since, 
for 
\code{n} observations there are \code{n-1} merges, there are \code{2^(n-1)} possible
orderings for the leaves in a cluster tree, or dendrogram.  The default
algorithm in \code{hc} is to order the subtree so that the tighter cluster is on
the left (the last, i.e. most recent, merge of the left subtree is at a lower 
value than the last
merge of the right subtree).  Observations are the tightest clusters possible,
and merges involving two observations place them in order by their 
observation sequence number.
}}
\section{Note}{
The squared Euclidean distance is used in the \code{movie} case, and in the case
when a data matrix is input. Note that the \code{movie} option returns 
invisibly.   
}
\section{Method}{
In the case of all methods when \code{bign = FALSE} (default), 
the Lance-Williams dissimilarity update formula is used to allow the 7 methods
to be implemented.   


In the direct data matrix case, 
a list of nearest neighbors is maintained, which 
allows the closest pair of clusters or observations to be determined by
scanning this list to find the pair of agglomerands on each iteration.
Hence \code{O(n)} operations are required for each of \code{n-1} iterations, i.e.
\code{O(n^2)} computation time in total.  The 
initial setting up of the \code{n} nearest neighbors also requires \code{O(n^2)} time.
The processing to be carried out following each agglomeration is \code{O(n)}.
In total, all methods implemented in \code{hc} are of time complexity \code{O(n^2)}.
Unless \code{bign = TRUE} (see below), storage 
complexity is \code{O(n^2)}.  Typical times on a Sun SPARCstation
1 for 400x4 and 800x4 arrays: 12 and 48 secs.


Ward's minimum variance method aims at finding compact, spherical clusters.
The complete link method finds similar clusters. The single link method,
which is closely related to the minimal spanning tree, adopts a 
\code{friends of friends} clustering strategy.  
The other methods can be regarded as aiming
for clusters with characteristics somewhere between the single and complete
link methods. 


In the case of the Ward method, and \code{bign = TRUE}, the nearest-neighbor 
chain algorithm is used, which gives an \code{O(n^2)}
computation time algorithm. The input matrix is stored, but not the
dissimilarities produced.  Computation time is marginally inferior to the
implementation of Ward's method when \code{bign = FALSE} (default).  Sample 
times for a 400x4 and an 800x4
array on a Sun SPARCstation 1: 12.5 and 50 secs.
}
\references{
A small sample: 


B. Everitt, 
Cluster Analysis
Heinemann Educ. Books, London 1974; 


P.H.A. Sneath and R.R. Sokal, 
Numerical Taxonomy
Freeman, San Francisco, 1973; 


M.R. Anderberg, 
Cluster Analysis for Applications
Academic Press, New York, 1973;


A.D. Gordon, 
Classification
Chapman and Hall, London, 1981; and


F. Murtagh, 
Multidimensional Clustering Algorithms
COMPSTAT Lectures 4,
Physica-Verlag, Wuerzburg, 1985 (for algorithmic details of algorithms used).
}
\seealso{
For finding assignments of objects to clusters, use \code{\link{members}}.
'Dist' is used for creating (dis)similarities.  Functions
\code{\link{cutree}}, and \code{\link{dist}} are already available in R.  Function \code{\link{members}}
is available in this function suite.
}
\examples{
data(iris)
iris <- as.matrix(iris[,1:4])
# Default Ward criterion
hierclust(iris, movie = TRUE)
h <- hierclust(iris)
k <- members(h)
}
\keyword{cluster}
\keyword{multivariate}
% Converted by Sd2Rd version 0.2-a5.
\eof
\name{logique}
\alias{logique}
\title{
Logical Coding
}
\usage{
logique(x)
}
\description{
  Simple logical coding of a vector: each value in the 
vector is replaced by a 1 (if it is above or equal to the median),
by a 0 (if it is below the median).
}
\arguments{
\item{x}{
real-valued vector, with no missing values.
}}
\value{
matrix of \code{length(a)} rows, and two columns.  The first column contains the
logically coded values of \code{a}, and the second column contains their 
complements.  Hence each row of this returned matrix necessarily sums to 1.
}
\section{Background}{
This form of coding is suitable for a subsequent correspondence analysis.
When all variable have been logically (or fuzzily) coded, the row masses 
(proportional to the row sums) are identical.  Logical coding results in
the input being in complete disjunctive form.
}
\references{
J.-P. Benzecri
Correspondence Analysis Handbook
Marcel Dekker, Basel, 1992.
}
\seealso{
\code{\link{flou}}, \code{\link{ca}}, \code{\link{supplr}}, \code{\link{supplc}}.  
}
\examples{
data(USArrests)
USArrests <- as.matrix(USArrests)
# Fuzzy coding of input variables
a.log <- logique(USArrests[,1])
b.log <- logique(USArrests[,2])
c.log <- logique(USArrests[,3])
d.log <- logique(USArrests[,4])
newdata <- cbind(a.log, b.log, c.log, d.log)
ca.newdata <- ca(newdata)
}
\keyword{multivariate}
\keyword{algebra}
% Converted by Sd2Rd version 0.2-a5.


\eof
\name{members}
\alias{members}
\title{
Cluster Memberships 
}
\usage{
members(a)
}
\description{
  From a succession of agglomerations produced by a hierarchical routine
in function `hierclust' or `hclust', determine cluster assignments of all 
objects, at all levels of the hierarchy.
}
\arguments{
\item{a}{
output produced by functions \code{hierclust} (or \code{hclust} or \code{mclust}).
}}
\value{
matrix of dimensions \code{n} by \code{n-2} giving cluster assignments to the 2, 3, \dots{}
n-1 cluster partitions of the hierarchy.  This corresponds to levels \code{n-2}
to 2.  The observations correspond to the rows, and are in
sequence, \code{1, 2, \dots{}}.  Column \code{j} specifies which of the \code{2, 3, \dots{} j-1}
clusters each observation is associated with at the level \code{j-1} of the 
agglomerative process.  The two clusters which merge in going from level 
\code{j-1} to \code{j-2} are replaced in the \code{j-2}nd column with the lower of the
two cluster sequence numbers.  
}
\section{Note}{


The time requirement of 'members' is \code{O(n^3)}.  Sample time for Sun 
SPARCstation 1, with 600 observations: 303 secs.
}
\seealso{
Functions \code{\link{hierclust}}, \code{\link{hclust}} produce the 
array containing the 
sequence of agglomerations which can be used as input for \code{\link{members}}.
}
\examples{
data(iris)
iris <- as.matrix(iris[,1:4])
h <- hierclust(iris, method=2)
k <- members(h)
}
\keyword{cluster}
\keyword{multivariate}
% Converted by Sd2Rd version 0.2-a5.
\eof
\name{partition}
\alias{partition}
\title{
Partitioning by Iterative Optimization
}
\usage{
partition(a, g, iter.max = 15, option = "mindst", diagnostics = FALSE)
}
\description{
  Returns cluster memberships.  In the case of a data matrix, the user 
specifies either a requested number of clusters; or - implicitly - as many 
clusters as there are rows in an initial guess at the cluster centers.  
Incorporates S function `kmeans'.
}
\arguments{
\item{a}{
matrix of multivariate data.
Each row corresponds to an observation, and each column corresponds to 
a variable.  
}
\item{g}{
Number of groups or clusters.
}
\item{centers}{
matrix of initial guesses for the cluster centers.  Each row represents a
cluster center, and thus \code{centers} must have the same number of columns as
\code{a}.  The number of rows in \code{centers} is the number of clusters that will be
formed.
}
\item{iter.max}{
maximum number of iterations.
}
\item{option}{
Either \code{mindst} (default) or \code{exch}.  Options for the Spaeth (1985) 
algorithm. In the former case, the variances of
the groups are optimized by assigning the objects to groups such that they
are minimally distant from group centers.  In the latter case, the variances
are optimized by exchanging the objects between groups such that they are
minimally distant from group centers. 
}
\item{diagnostics}{
FALSE (default) implies that cluster cardinalities, cluster center coordinates,
and cluster compactness values will not be output to the command window.  
}}
\value{
in the case of the Hartigan algorithm (where initial cluster centers 
are provided) a list with the following components:

\item{cluster}{
vector of integers, ranging from \code{1} to \code{ng} (or number of rows in 
\code{centers}), with length the same as the number of rows of \code{a}.  
The \code{i}th value indicates the group in which the
\code{i}th observation belongs.
}
\item{centers}{
matrix containing the coordinates of the final group centers.
}
\item{withinss}{
vector of length \code{ng} or number of rows in \code{centers}.  The \code{i}th value 
gives the within group sum of squares for the \code{i}th group.
}
\item{size}{
vector of length \code{ng}, or the number of rows in \code{centers}.  
The \code{i}th value gives the number of observations in group \code{i}.


In the case of the Spaeth algorithm (where the number of clusters is given),
the first of these, only, is returned.  
}}
\section{Method}{
Consider first the clustering of coordinate data.
The object is to find a partition of the observations with \code{ng} (or the 
number of rows in \code{centers}) groups that
minimizes the sum of \code{withinss} values.  To actually guarantee the 
minimum would be computationally
infeasible in many settings; this function finds a local minimum, that is,
a solution such that there is no single switch of an observation from one
group to another group that will decrease the objective criterion.  In the
case of the Spaeth (1985) algorithms, the local
minimum arrived at is dependent on the initial arbitrary assignment of 
observations to groups.  This arbitrary assignment is carried out using a
random number generator.  Subsequent executions of the \code{partition} function
will make use of different initial arbitrary assignments.  The function
\code{partition} should therefore be run a number of times, and the best result
kept.


The initial data is not standardized: it may be advisable to divide all column
values by the range of the column's values, or to standardize in some other 
way.


Note that this routine is best used by specifying a small number of groups;
specifying a large number of groups may cause many empty classes.  When this
happens, group centroids and compactness values may be undefined.  This may
also happen when there are very large values in the input data set.  
Cluster memberships, though, can still be used.  


Sample timings for the Spaeth (1985) algorithms (accessed by specifying the 
desired number of groups): 33000 rows, 4 columns, 3 groups took about 40 
secs. on a Sun SPARCstation 1; 50 groups took about 140 secs.  


When deciding on the number of clusters, Hartigan (1975, pp. 90-91) suggests
the following rough rule of thumb.  If \code{k} is the result of \code{partition} with
\code{k} groups, and \code{kplus1} is the result with \code{k+1} groups, then it is 
justifiable to add the extra group when 


(sum(k\$withinss)/sum(kplus1\$withinss)-1)*(nrow(a)-k+1)


is greater than 10.


When a partition is being obtained from a dendrogram, the procedure is
straightforward: a slice is made, orthogonal to the cluster criterion value
axis, and the clusters read off.
}
\references{
Spaeth, H. (1985).  
Cluster Dissection and Analysis: Theory, Fortran Programs, Examples.
Ellis Horwood, Chichester.


Hartigan, J.A. (1975).
Clustering Algorithms.
Wiley, New York.


Hartigan, J.A. and Wong, M.A. (1979).
A k-means clustering algorithm,
Applied Statistics,
vol. 28, pp. 100-108.
}
\seealso{
\code{\link{kmeans}} (incorporated into this function); \code{\link{hierclust}}
(hierarchical clustering routines, incorporating \code{\link{hclust}}).  
}
\examples{
data(iris3)
iris <- rbind(iris3[,,1], iris3[,,2], iris3[,,3])
pp <- partition(iris,3)
# Plot the results in the plane comprising the first two columns of `a'
x <- iris[,1]       # for convenience
y <- iris[,2]
plot(x, y, type="n")     # set up plot scales, etc.
points(x[pp==1], y[pp==1],pch="*")
points(x[pp==2], y[pp==2],pch="+")
points(x[pp==3], y[pp==3],pch="o")


# Secondly, specifying guesses at cluster centers.
irismean <- t(apply(iris3, c(2, 3), 'mean'))
library(mva)
km <- partition(iris, irismean)
wrong <- km$cluster!=rep(1:3, c(50, 50, 50))
plot(iris[,2], iris[,3], type="n")
text(iris[!wrong, 2], iris[!wrong, 3], km$cluster)
# identify cluster membership that is correct
points(iris[wrong, 2], iris[wrong, 3], pch=15, col=2)
# boxes for points in error
title(main="K-Means Clustering of the Iris Data")
}
\keyword{cluster}
\keyword{multivariate}
% Converted by Sd2Rd version 0.2-a5.


\eof
\name{pca}
\alias{pca}
\title{Principal Components Analysis}
\description{
  Finds a new coordinate system for multivariate data such that the
  first coordinate has maximal variance, the second coordinate has
  maximal variance subject to being orthogonal to the first, etc.
}
\usage{
pca(a, method=3)
}
\arguments{
  \item{a}{data matrix to be decomposed, the rows representing
    observations and the columns variables.  Missing values are not
    supported.}
  \item{method}{integer taking values between 1 and 8.  \code{method} =
    1 implies no  
transformation of data matrix.  Hence the singular value decomposition (SVD) 
is carried out on a sums of squares and 
cross-products matrix.  \code{method} = 2 implies that the observations are centered
to zero mean.  Hence the SVD is carried out on a variance-covariance matrix.
\code{method} = 3 (default) implies that the observations are centered to zero 
mean, and additionally reduced to unit standard deviation.  In this case the 
observations are standardized.  Hence the SVD is carried out on a correlation
matrix. \code{method} = 4 implies that the observations are normalized by being 
range-divided, and then the variance-covariance matrix is used.  \code{method} = 5
implies that the SVD is carried out on a Kendall (rank-order) correlation
matrix.  \code{method} = 6 implies that the SVD is carried out on a Spearman 
(rank-order) correlation matrix.  \code{method} = 7 implies that the SVD is carried
out on the sample covariance matrix.  \code{method} = 8 implies that the SVD is
carried out on the sample correlation matrix.
}}
\value{
  list describing the principal components analysis:

  \item{rproj}{projections of row points on the new axes.}
  \item{cproj}{projections of column points on the new axes.}
  \item{evals}{eigenvalues associated with the new axes.  These provide
    figures of merit for the \code{variance explained} by the new axes.
    They are usually quoted in terms of percentage of the total, or in
    terms of cumulative percentage of the total.}
  \item{evecs}{eigenvectors associated with the new axes.  This
    orthogonal matrix describes the rotation.  The first column is the
    linear combination of columns of \code{a} defining the first
    principal component, etc.}
}
\section{Side Effects}{
  When carrying out a PCA of a hierarchy object, the partition is
  specified bt \code{lev}.  The level plus the associated number of
  groups equals the number of observations, at all times.
}
\section{Note}{
In the case of \code{method} = 3, if any column point has zero standard deviation,
then a value of 1 is substituted for the standard deviation.


Up to 7 principal axes are determined.  The inherent dimensionality of either
of the dual spaces is ordinarily \code{min(n,m)} where \code{n} and \code{m} are respectively
the numbers of rows and columns of \code{a}.  The centering transformation which is
part of \code{method}s 2 and 3 introduces a linear dependency causing the inherent
dimensionality to be \code{min(n-1,m)}.  Hence the number of columns returned in
\code{rproj}, \code{cproj}, and \code{evecs} will be the lesser of this inherent 
dimensionality and 7.


In the case of \code{methods} 1 to 4, very small negative eigenvalues, if they 
arise, are an artifact of the SVD algorithm used, and may be treated as zero. 
In the case of PCA using rank-order correlations (\code{methods} 5 and 6), negative
eigenvalues indicate that a Euclidean representation of the data is not
possible.  The approximate Euclidean representation given by the axes 
associated with the positive eigenvalues can often be quite adequate for
practical interpretation of the data.


Routine \code{prcomp} is identical, to within small numerical
precision differences, to \code{method} = 7 here.  The examples below show
how to transform the outputs of the present implementation (\code{pca}) onto 
outputs of \code{prcomp}.


Note that a very large number of columns in the input data matrix will
cause dynamic memory problems: the matrix to be diagonalized requires
O(\eqn{m^2}) storage where \eqn{m} is the number of variables.
}
\section{Method}{
  A singular value decomposition is carried out. 
}
\section{Background}{
  Principal components analysis defines the axis which provides the best
  fit to both the row points and the column points.  A second axis is
  determined which best fits the data subject to being orthogonal to the
  first.  Third and subsequent axes are similarly found.  Best fit is in
  the least squares sense.  The criterion which optimizes the fit of the
  axes to the points is, by virtue of Pythagoras' theorem,
  simultaneously a criterion which optimizes the variance of projections
  on the axes.

  Principal components analysis is often used as a data reduction
  technique.  In the pattern recognition field, it is often termed the
  Karhunen-Loeve expansion since the data matrix \code{a} may be written
  as a series expansion using the eigenvectors and eigenvalues found.
}
\references{
  Many multivariate statistics and data analysis books include a
  discussion of principal components analysis.  Below are a few examples: 

  C. Chatfield and A.J. Collins, \code{Introduction to Multivariate
    Analysis}, Chapman and Hall, 1980 (a good, all-round introduction); 

  M. Kendall, \code{Multivariate Analysis}, Griffin, 1980 (dated in
  relation to computing techniques, but exceptionally clear and concise
  in the treatment of many practical aspects); 

  F.H.C. Marriott, \code{The Interpretation of Multiple Observations},
  Academic, 1974 (a short, very readable textbook); 

  L. Lebart, A. Morineau, and K.M. Warwick, `Multivariate Descriptive
  Statistical Analysis', Wiley, 1984 (an excellent geometric treatment
  of PCA); 

  I.T. Joliffe, \code{Principal Component Analysis}, Springer, 1980.
}
\seealso{
  \code{\link{svd}}, \code{\link{prcomp}}, \code{\link{cancor}}.
}
\examples{
data(iris)
iris <- as.matrix(iris[,1:4])
pcprim <- pca(iris)
# plot of first and second principal components
plot(pcprim$rproj[,1], pcprim$rproj[,2])
# variance explained by the principal components
pcprim$evals*100.0/sum(pcprim$evals)
# In the implementation of the S function `prcomp', different results are
# produced.  Here is how to obtain these results, using the function `pca'.
library(mva)
# Consider the following result of `prcomp':
old <- prcomp(iris)
# With `pca', one would do the following:
new <- pca(iris, method=7)
}
\keyword{multivariate}
\keyword{algebra}
% Converted by Sd2Rd version 0.2-a5.
\eof
\name{sammon}
\alias{sammon}
\title{
Sammon's Non-Linear Mapping
}
\description{
  Finds a new, reduced-dimensionality, coordinate system for multivariate data 
such that the an error criterion between distances in the given space, 
and distances in the result space, is minimized.
}
\usage{
sammon(a, p=2, maxit=100, tol=0.05, alpha=0.3, diagnostics=FALSE)
}
\arguments{
  \item{a}{
    input matrix, coordinate data. No missing values.
  }
  \item{p}{
    dimensionality of output space.
  }
  \item{maxit}{maximum number of iterations}
  \item{tol}{
    tolerance on error criterion.
  }
  \item{alpha}{
    step size for gradient descent optimization.
  }
  \item{diagnostics}{
    whether or not error value is output at each step of the iterative 
    optimization.
}}
\value{
projections in the new space.

\item{rproj}{
matrix of projections of the row observations, as yielded by the nonlinear
mapping algorithm.
}}
\section{Note}{
It may be useful to run this routine a number of times and to keep the
result yielding the smallest error.  This mapping error, and the number of
iterations required for convergence, are output to the command window.
}
\references{
W. Siedlecki, K. Siedlecka and J. Sklansky, 
An overview of mapping techniques for exploratory pattern analysis, 
Pattern Recognition,
21, 411-429, 1988.


J.W. Sammon, 
A nonlinear mapping for data structure analysis,
IEEE Trans. Computers, C-18, 401-409, 1969.
}
\examples{
data(iris)
iris <- as.matrix(iris[,1:4])  
mds <- sammon(iris, tol=0.05, maxit=200)
# Now plot observations 1-50, 51-100 and 101-150 distinctively;
# add a set of axes through x=0 and y=0.
plot(mds$rproj[,1], mds$rproj[,2], type="n", xlab="Axis 1", ylab="Axis 2",
main="2-d Sammon mapping of iris data")
points(mds$rproj[1:50,1], mds$rproj[1:50,2], pch="*")
points(mds$rproj[51:100,1], mds$rproj[51:100,2], pch="+")
points(mds$rproj[101:150,1], mds$rproj[101:150,2], pch="o")
}
\keyword{multivariate}
% Converted by Sd2Rd version 0.2-a5.




\eof
\name{supplc}
\alias{supplc}
\title{
Supplementary Columns in Correspondence Analysis
}
\description{
  Using the results of a correspondence analysis, project new columns into the
  factor space.
}
\usage{
supplc(a, ca.res)
}
\arguments{
\item{a}{
data matrix to be projected.  Must have same number of rows as matrix which
was initially input to the correspondence analysis.
}
\item{ca.res}{
the output of a correspondence analysis.  The following components of this
object are used: \code{evals}, \code{rproj} and \code{cproj}.
}}
\value{
a list containing the matrix \code{proj}, 
projections of the columns of \code{a} on the correspondence analysis factors.
}
\references{
See function \code{ca}.
}
\seealso{
Correspondence analysis: \code{\link{ca}}. 
Supplementary rows and columns: \code{\link{supplr}}, \code{\link{supplc}}.  Initial data coding:
\code{\link{flou}}, \code{\link{logique}}.  Other functions producing objects of class "reddim":
\code{\link{pca}}, \code{\link{sammon}}.  Other related functions: \code{\link{prcomp}}, \code{\link{cancor}}, \code{\link{cmdscale}}.
}
\examples{
data(USArrests)
USArrests <- as.matrix(USArrests)
corr <- ca(USArrests[,1:2])
newproj <- supplc(USArrests[,3:4], corr)
# plot of first and second factors, and of supplementary columns:
plot(corr$cproj[,1], corr$cproj[,2],type="n")
text(corr$cproj[,1], corr$cproj[,2])
points(newproj$proj[,1], newproj$proj[,2], col=2)
}
\keyword{multivariate}
\keyword{algebra}
% Converted by Sd2Rd version 0.2-a5.


\eof
\name{supplr}
\alias{supplr}
\title{
Supplementary Rows in Correspondence Analysis
}
\description{
  Using the results of a correspondence analysis, project new rows into the
  factor space.
}
\usage{
supplr(a, ca.res)
}
\arguments{
\item{a}{
data matrix to be projected.  Must have same number of columns as matrix which
was initially input to the correspondence analysis.
}
\item{ca.res}{
the output of a correspondence analysis.  The following components of this
object are used: \code{evals}, \code{rproj} and \code{cproj}.
}}
\value{
a list containing the matrix \code{proj}, 
projections of the rows of \code{a} on the correspondence analysis factors.
}
\references{
See function \code{ca}.
}
\seealso{
Correspondence analysis: \code{\link{ca}}. 
Supplementary rows and columns: \code{\link{supplr}}, \code{\link{supplc}}.  Initial data coding:
\code{\link{flou}}, \code{\link{logique}}.  Other functions producing objects of class \code{reddim}:
\code{\link{pca}}, \code{\link{sammon}}.  Other related functions: \code{\link{prcomp}}, \code{\link{cancor}}, \code{\link{cmdscale}}.
}
\examples{
data(USArrests)
USArrests <- as.matrix(USArrests)
corr <- ca(USArrests[1:25,])
newproj <- supplc(USArrests[26:50,], corr)
# plot of first and second factors, and of supplementary columns:
plot(corr$cproj[,1], corr$cproj[,2],type="n")
text(corr$cproj[,1], corr$cproj[,2])
points(newproj$proj[,1], newproj$proj[,2], col=2) 
}
\keyword{multivariate}
\keyword{algebra}
% Converted by Sd2Rd version 0.2-a5.


\eof
